﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.InvoiceApply.Current.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.InvoiceApply',
    xtype: 'App.View.Kivii.Finances.InvoiceApply.Current.List',
    dataSource: '/Restful/Kivii.Finances.Entities.InvoiceApply/QueryEx.json',
    title: '我的申请',
    iconCls: 'x-fa fa-circle-o-notch',
    filter: {
    	OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
        OperateTimeGreaterThanOrEqualTo: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        OperateTimeLessThan: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1)
    },
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 100,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'status',
                // value: 50,
                width:130,
                editable: false,
                filterPickList: true,
                reference: 'status',
                store: {
                    fields: ['displayInfo', 'status'],
                    data: [{
                        displayInfo: '未开票',
                        status: 50
                    }, {
                        displayInfo: '已开票',
                        status: 51
                    }]
                },
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanStatus'
                    }
                },
                listeners: {
                    select: '_onSelectStatus'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '申请日期 开始',
                format: 'Y-m-d',
                labelWidth: 90,
                width:220,
                reference: 'startTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                width:160,
                reference: 'endTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,OperatorName',
                emptyText: '模糊搜索流水号、付款单位、申请人',
            }, {
                text: '借票申请',
                iconCls: 'x-fa fa-circle-o-notch',
                type: 'Debit',
                handler: '_onCreateApply'
            }, {
                text: '撤销',
                iconCls: 'x-fa fa-rotate-left',
                iconAlign: 'left',
                handler: '_onRejectApply',
                bind: {
                    disabled: '{!_selection}'
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            width: 200
        }, {
            xtype: 'datecolumn',
            text: '申请日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 120
        }, {
            text: '申请金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 110
        }, {
            text: '申请人',
            dataIndex: 'OperatorName',
            renderer: '_renderOperatorName',
            width: 140
        }, {
            text: '申请类型',
            dataIndex: 'OperateType',
            renderer: '_renderOperateType',
            width: 100
        }, {
            text: '状态',
            dataIndex: 'Status',
            renderer: '_renderStatus',
            width: 100
        }, {
            text: '税号',
            dataIndex: 'PayerTaxNumber',
            width: 200
        }, {
            text: '地址',
            dataIndex: 'PayerCompanyAddress',
            renderer: '_renderPayerCompanyAddress',
            width: 260
        }, {
            text: '开户行',
            dataIndex: 'PayerRegisteredBank',
            renderer: '_renderPayerRegisteredBank',
            width: 260
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            minWidth: 200,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderOperatorName: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OwnerName')) return _value + '(' + _record.data.OwnerName + ')';
            return _value;
        },
        _renderOperateType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Payment") {
                _meta.style = 'background-color:#D1FCA5';
                if (_record.data.Type === 'VAT') return "收款(普票)";
                return "收款";
            }
            if (_value === "Debit") {
                _meta.style = 'background-color:#F3F781';
                if (_record.data.Type === 'VAT') return "借票(普票)";
                return "借票";
            }
            if (_value === "Cash") {
                if (_record.data.Type === 'VAT') return "现金(普票)";
                return "现金";
            }
            if (_value === "Pos") {
                if (_record.data.Type === 'VAT') return "刷卡(普票)";
                return "刷卡";
            }
            if (_value === "AliPay") {
                if (_record.data.Type === 'VAT') return "支付宝(普票)";
                return "支付宝";
            }
            if (_value === "WeChat") {
                if (_record.data.Type === 'VAT') return "微信(普票)";
                return "微信";
            }
            if (_record.data.Type === 'VAT') return _value + "(普票)";
            return _value;
        },
        _renderStatus: function(_value, _meta, _record, _rowIndex) {
            if (_value <= 25) {
                _meta.style = 'background-color:#D5F5E3';
                return "审批中...";
            }
            if (_value === 50) {
                _meta.style = 'background-color:#D6EAF8';
                return "已批准";
            }
            if (_value === 51) {
                _meta.style = 'background-color:#D6EAF8';
                return "开票列队中";
            }
            if (_value === 52) {
                _meta.style = 'background-color:#D6EAF8';
                return "开票中";
            }
            if (_value === 55 || _value === 56) {
                _meta.style = 'background-color:#F5B7B1';
                return "开票失败";
            }
            if (_value === 57) {
                _meta.style = 'background-color:#F5B7B1';
                return "驳回";
            }
            if (_value === 58) {
                _meta.style = 'background-color:#F5B7B1';
                return "发票已作废";
            }
            if (_value === 59) {
                _meta.style = 'background-color:#F5B7B1';
                return "申请已合并";
            }
            if (_value === 60) {
                _meta.style = 'background-color:#F5B7B1';
                return "红字冲销";
            }
            if (_value === 2147483647) {
                _meta.style = 'background-color:#ABEBC6';
                return "已开具";
            }
            return "未处理";
        },
        _renderPayerCompanyAddress: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerPhone;
        },
        _renderPayerRegisteredBank: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerBankAccount;
        },

        _onSelectStatus: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            delete _view.filter.StatusGreaterThanOrEqualTo;
            delete _view.filter.StatusLessThanOrEqualTo;
            if (_record.data.status === 50) {
                _view.filter.StatusLessThanOrEqualTo = _record.data.status;
            }
            if (_record.data.status === 51) {
                _view.filter.StatusGreaterThanOrEqualTo = _record.data.status;
            }
            _view.reload();
        },
        _cleanStatus: function(_ctrl) {
            _ctrl.setValue('');
            let _controller = this;
            let _view = _controller.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.StatusGreaterThanOrEqualTo;
            delete _view.filter.StatusLessThanOrEqualTo;
            _view.reload();
        },
        _onCreateApply: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _apply = {};
            _apply.Metadata = {};
            _apply.PaymentKvids = [];
            _apply.OperateType = _ctrl.type;
            _apply.Amount=0;
            _apply.PayerName=null;
            _apply.PayeeName = KiviiContext.CurrentMember.OrganizationName;
            _apply.PayeeTaxNumber = null;
            _apply.OperateTime = new Date();
            _apply.Currency = 'CNY';

            let _form = Ext.create('App.Control.Kivii.Finances.InvoiceApply.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 900,
                height: 450,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_apply, _ctrl);
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _onRejectApply: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _applyKvids = [];
            let _select = _vm.data.selections || [];
            _select.forEach(function(item) {
                _applyKvids.push(item.data.Kvid);
            });
            Ext.MessageBox.show({
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                msg: '请填写原因:',
                width: 300,
                buttons: Ext.MessageBox.OKCANCEL,
                multiline: true,
                scope: this,
                fn: this._showResultText,
                animateTarget: _ctrl
            });

        },
        _showResultText: function(_btn, _text, _ctrl) {
            if (_btn !== 'ok') return;
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _applyKvids = [];
            let _select = _vm.data.selections || [];
            for (let _i = 0; _i < _select.length; _i++) {
                let _item = _select[_i].data;
                _applyKvids.push(_item.Kvid);
            }
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.InvoiceApply/Reject.json',
                jsonData: {
                    ApplyKvids: _applyKvids,
                    Summary: _text
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('驳回成功！');
                    _view.reload();
                },
                failure: function(_response) {
                }
            });
        }
    }
});