﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "发票统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceStatistic : RestfulExecution<Invoice>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool Offseted { get; set; }

        /// <summary>
        /// 用于查询借票是否到账使用 
        /// </summary>
        public bool? Payed { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Invoice>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.RootKvid == o.Kvid);
            if (Offseted)//只查作废或者红字的发票
            {
                sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime && o.Amount < 0);
            }
            else
            {
                //查当月的全部发票(包含当月到账的借票)
                sqlExpress.And(o => ((o.OperateTime >= BeginTime && o.OperateTime < EndTime) || (o.PayedTime.Value >= BeginTime && o.PayedTime.Value < EndTime && o.OperateTime < BeginTime)));
                if (Payed != null && Payed.Value) sqlExpress.And(o => o.AmountPayment == o.Amount);
                if (Payed != null && !Payed.Value) sqlExpress.And(o => o.AmountPayment < o.Amount);
            }
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            return rtns;
        }
    }

    [Api(Description = "结算统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementStatistics : RestfulExecution<Settlement>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool? Payed { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Settlement>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            //sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
            if (Payed != null)
            {
                if (Payed.Value) sqlExpress.And(o => o.Amount == o.AmountPayment);
                else sqlExpress.And(o => o.Amount > o.AmountPayment);
            }
            sqlExpress.And(o => o.CreateTime >= BeginTime && o.CreateTime < EndTime);

            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            return rtns;
        }
    }
}
